/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | www.openfoam.com
     \\/     M anipulation  |
-------------------------------------------------------------------------------
    Copyright (C) 2011-2014 OpenFOAM Foundation
    Copyright (C) 2020 OpenCFD Ltd.
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Global
    alphaCourantNo

Description
    Calculates and outputs the mean and maximum Courant Numbers.

\*---------------------------------------------------------------------------*/

scalar maxAlphaCo
(
    runTime.controlDict().get<scalar>("maxAlphaCo")
);

scalar maxAlphaDdt
(
    runTime.controlDict().getOrDefault("maxAlphaDdt", GREAT)
);

scalar maxDi
(
    runTime.controlDict().getOrDefault<scalar>("maxDi", GREAT)
);

scalar alphaCoNum = 0.0;
scalar meanAlphaCoNum = 0.0;
scalar ddtAlphaNum = 0.0;
scalar DiNum = 0.0;

if (mesh.nInternalFaces())
{
    scalarField sumPhi
    (
        fluid.nearInterface()().internalField()
       *fvc::surfaceSum(mag(phi))().internalField()
    );

    alphaCoNum = 0.5*gMax(sumPhi/mesh.V().field())*runTime.deltaTValue();

    meanAlphaCoNum =
        0.5*(gSum(sumPhi)/gSum(mesh.V().field()))*runTime.deltaTValue();

    ddtAlphaNum = fluid.ddtAlphaMax().value()*runTime.deltaTValue();

    DiNum = fluid.maxDiffNo();
}

Info<< "Interface Courant Number mean: " << meanAlphaCoNum
    << " max: " << alphaCoNum << endl;

Info<< "Maximum ddtAlpha : " << ddtAlphaNum << endl;

Info<< "Maximum DiffNum : " << DiNum << endl;


// ************************************************************************* //
